/*
  composite.cc

  Copyright (C) 2015 Taiji Yamada <taiji@aihara.co.jp>
*/
#include <iostream>
#include <string>
#include <libgen.h>
#include <vector>
#include <algorithm>
#include <cctype>
#include <cmath>
#include <boost/lexical_cast.hpp>
#include "prime_util.hh"

int usage(std::ostream &os, int argc, char *argv[])
{
  (void)argc;
  os << "Usage:\n\t" << basename(argv[0]) << " [-c number ...] [-v number ...] [-w number ...] [-m times]\n";
  return os == std::cerr ? 1 : 0;
}
int main(int argc, char *argv[])
{
  std::vector<isprime_bit_table_class::huge_uint_type> constants, times, composites;
  std::vector<bool> zero;
  int a;
  size_t i, j, k, m = 5;

  for (a=1; a<argc; ++a)
    if (std::string(argv[a]) == "-h")
      return usage(std::cerr, argc, argv);
    else if (std::string(argv[a]) == "-c" && ++a < argc)
      for (; a<argc; ++a) {
	isprime_bit_table_class::huge_uint_type n;
	try { n = boost::lexical_cast<isprime_bit_table_class::huge_uint_type>(argv[a]); }
	catch (...) { --a; break; }
	constants.push_back(n);
      }
    else if (std::string(argv[a]) == "-v" && ++a < argc)
      for (; a<argc; ++a) {
	isprime_bit_table_class::huge_uint_type n;
	try { n = boost::lexical_cast<isprime_bit_table_class::huge_uint_type>(argv[a]); }
	catch (...) { --a; break; }
	times.push_back(n);
	zero.push_back(false);
      }
    else if (std::string(argv[a]) == "-w" && ++a < argc)
      for (; a<argc; ++a) {
	isprime_bit_table_class::huge_uint_type n;
	try { n = boost::lexical_cast<isprime_bit_table_class::huge_uint_type>(argv[a]); }
	catch (...) { --a; break; }
	times.push_back(n);
	zero.push_back(true);
      }
    else if (std::string(argv[a]) == "-m" && a+1 < argc)
      m = atoll(argv[++a]);
    else
      break;

  isprime_bit_table_class::huge_uint_type c(1);
  for (k=0; k<constants.size(); ++k)
    c *= constants[k];

  size_t t(1);
  for (k=0; k<times.size(); ++k)
    t *= m;

  j = 0;
  for (i=0; i<t; ++i) {
    isprime_bit_table_class::huge_uint_type n(c);
    for (k=0; k<times.size(); ++k) {
      isprime_bit_table_class::huge_uint_type p;
      size_t d = (i % (size_t)std::pow((double)m, (double)(k+1))) /
	(size_t)std::pow((double)m, (double)k);
      p = pow(times[k], (isprime_bit_table_class::huge_uint_type)(zero[k] ? d : d+1));
      n *= p;
    }
    composites.push_back(n);
  }

  sort(composites.begin(), composites.end());

  for (k=0; k<composites.size(); ++k)
    std::cout << composites[k] << std::endl;

  return 0;
}
